#!/usr/bin/env python
# -*- coding: utf-8  -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2012 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################

"""
``rattail.pyramid.views.departments`` -- Department Views
"""

import transaction
from pyramid.httpexceptions import HTTPFound
from pyramid.view import view_config

from edbob.pyramid import filters
from edbob.pyramid import forms
from edbob.pyramid import grids
from edbob.pyramid import Session

import rattail


@view_config(route_name='departments.list', renderer='/departments/index.mako')
def list_departments(context, request):

    fmap = filters.get_filter_map(
        rattail.Department,
        exact=['number'],
        ilike=['name'])

    config = filters.get_search_config(
        'departments.list', request, fmap,
        include_filter_name=True,
        filter_type_name='lk')

    search = filters.get_search_form(config)

    config = grids.get_grid_config(
        'departments.list', request, search,
        filter_map=fmap, sort='name', deletable=True)

    smap = grids.get_sort_map(
        rattail.Department,
        ['number', 'name'])

    def query(config):
        q = Session.query(rattail.Department)
        q = filters.filter_query(q, config)
        q = grids.sort_query(q, config, smap)
        return q

    departments = grids.get_pager(query, config)
    g = forms.AlchemyGrid(
        rattail.Department, departments, config,
        gridurl=request.route_url('departments.list'),
        delurl='department.delete',
        )

    g.configure(
        include=[
            g.number,
            g.name,
            ],
        readonly=True)

    grid = g.render(class_='hoverable departments')
    return grids.render_grid(request, grid, search)


@view_config(route_name='department.delete')
def delete_department(context, request):
    uuid = request.matchdict['uuid']
    dept = Session.query(rattail.Department).get(uuid) if uuid else None
    assert dept
    with transaction.manager:
        q = Session.query(rattail.Product)
        q = q.filter(rattail.Product.department_uuid == dept.uuid)
        if q.count():
            q.update({'department_uuid': None}, synchronize_session=False)
        Session.delete(dept)
    return HTTPFound(location=request.route_url('departments.list'))


def includeme(config):
    config.add_route('departments.list', '/departments')
    config.add_route('department.delete', '/department/{uuid}/delete')
    config.scan(__name__)
